function [out Fout f_ax] = designBH(A,T,nLen,f0)
% function [out Fout f_ax] = designBH(A,T,nLen,f0)
%
% Design a Blackman-Harris window source for FDTD simulation:
%       A       - Pulse amplitude
%       T       - Sample period (1/fs)
%       nLen    - Length of pulse (samples)
%       f0      - Normalised cutoff frequency (0.5=Nyquist)
%
% Returns:
%       out     - Pulse in discrete time domain
%       Fout    - DFT Magnitude of pulse (dB)
%       f_ax    - Frequency axis (normalised) for Fout
%
% Example:
%       [out Fout f_ax] = designBH(1,1/44100,128,0.25);
%
% Based on:
% J. Sheaffer, M. van Walstijn, and B. M. Fazenda. 
% "Physical and numerical constraints in source modelling for finite 
%   difference simulation of room acoustics."
%
% Code by Jonathan Sheaffer, University of Salford, March 2013
% contact: jon@jonsh.net
%
ad=[0.353222222 -0.488 0.145 -0.010222222];

for n=0:nLen-1
    if n<(1.55/f0)
        out(n+1)=0;
        for ii=0:3
            out(n+1)=out(n+1)+ad(ii+1)*cos(2*pi*f0*n*ii/1.55);
        end
    else
        out(n+1)=0;
    end
end

out=A*out/norm(out,inf);

Fout=20*log10(abs(fft(out)));
Fout(length(Fout)/2+1:end)=[];
f_ax=linspace(0,0.5,length(Fout));


 


end