function [out Fout f_ax] = designSineGaussian(A,T,nLen,n0,fc,f0)
% function [out Fout f_ax] = designSineGaussian(A,T,nLen,n0,fc,f0)
%
% Design a Sine-Modulated Gaussian pulse for FDTD simulation:
%       A       - Pulse amplitude
%       T       - Sample period (1/fs)
%       nLen    - Length of pulse (samples)
%       n0      - Time shifting (samples)
%       fc      - Normalised cutoff frequency (0.5=nyquist)
%       f0      - Normalised modulation frequency (0.5=nyquist)
%
% Returns:
%       out     - Pulse in discrete time domain
%       Fout    - DFT Magnitude of pulse (dB)
%       f_ax    - Frequency axis (normalised) for Fout
%
% Example:
%       [out Fout f_ax] = designSineGaussian(1,1/44100,1024,50,0.25,0.05)
%
% Based on:
% J. Sheaffer, M. van Walstijn, and B. M. Fazenda. 
% "Physical and numerical constraints in source modelling for finite 
%   difference simulation of room acoustics."
%
% Code by Jonathan Sheaffer, University of Salford, March 2013
% contact: jon@jonsh.net
%
sigma=sqrt(2*log(2))/(2*pi*(fc/T));
n=0:nLen-1;
out=exp(-T^2*(n-n0).^2/(2*sigma^2)).*sin(2*pi*f0*(n-n0));
out=A*out/norm(out,inf);
Fout=20*log10(abs(fft(out)));
Fout(length(Fout)/2+1:end)=[];
f_ax=linspace(0,0.5,length(Fout));

end