function out = pcsDesign(M,f0,Q,maxF,fc,T)

% Design a Physically-Constrained Source for FDTD Simulation
% with MAXFLAT as pulse shaping filter, and sphere model as mechanical
% filter.
%
% function out = pcsDesign(M,f0,Q,maxF,fc,T)
%
% Inputs:
%   M       - Mass of sphere (kg)
%   f0      - Normalised low resonance of mechanical system (0.5=Nyquist)
%   Q       - Q of mechanical system
%   maxF    - Maximum driving force of sphere (Newtons)
%   fc      - Normalised cutoff frequency pulse (0.5=Nyquist)
%   T       - Temporal sample period (s)
%
% Output:
%   out     - Surface velocity of source to be injected
%
% Usage example in FDTD model:
%   v = pcsDesign (25e-3,0.03,0.7,250e-3,0.25,T);
%   For Yee schemes: 
%       p(l,m,i) = p(l,m,i) + g0*v(n)
%   For Wave equation schemes:
%       p(l,m,i) = p(l,m,i) + (g0/(2T))*(v(n)-v(n-2))
%
%  where g0 is given by Equations 37 and 40, respectively.
%  p is pressure at time (n+1), and v(n) is surface velocity at time (n+1).
%
% Notes:
%   1. Default MAXFLAT order is N=16 (edit "N" to change)
%   2. Default length of output function is 4096 samples (edit "hLen" to change)
%   3. Default source size equals one grid node (edit "A_s" to change)
%
% Based on:
% J. Sheaffer, M. van Walstijn, and B. M. Fazenda. 
% "Physical and numerical constraints in source modelling for finite 
%   difference simulation of room acoustics."
%
% Code by Jonathan Sheaffer, University of Salford, March 2013
% contact: jon@jonsh.net
%
fs=1/T;
    N=16;           % Maxflat IR taps
    hLen = 4096;    % Full IR Length limit

    % PULSE-SHAPING FILTER
    % Generate tap coefficients for MAXFLAT lowpass filter
    % based on Khan & Ohba, Electronics Letters Vol 36 No 23, 2000
    % ***************************************************************
    h=zeros(1,N);                   % Impulse Response
    Qn=(2*N-1);
    for n=-Qn:Qn
        top=factdbl(Qn)^2 * sin(n*2*pi*fc);
        if mod(n,2)                 % Figure out if n is odd or even
            bot=2*n*factdbl(2*N+n-1)*factdbl(2*N-n-1);
        else
            bot=pi*n*factdbl(2*N+n-1)*factdbl(2*N-n-1);
        end
        h(n+Qn+1)=top/bot;
    end
    h(Qn+1)=2*fc;                  
    h=(h/norm(h,inf))*maxF;         % Normalise and scale
    h(end:hLen)=0;                  % Pad Maxflat IR

    % MECHANICAL FILTER
    % Simulate the dynamics of a small pulsating sphere
    % ***************************************************************
    w0 = 2*pi*f0*fs;                % Angular Resonance
    K = M*w0^2;                     % Elasticity
    R = w0*M/Q;                     % Resistance
    beta=w0/tan(w0*T/2);            % Bilinear operator
    b0=beta/(M*beta^2+R*beta+K);    % Transform coefficients
    b2=-b0;
    a1=(2*(K-M*beta^2))/(M*beta^2+R*beta+K);
    a2=1-(2*R*beta/(M*beta^2+R*beta+K));
    num=[b0 0 b2];
    den=[1 a1 a2];
    out=filter(num,den,h);          % Surface Velocity

end

function out=factdbl(in)            % Calculate a double factorial
    out = 1;
    for ii = in:-2:1
        out = out * ii;
    end
end